/* 
 * Copyright (c) 2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.doc;

import javax.security.auth.login.LoginContext;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDescriptionDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDescriptionType;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.security.KeystoreLoginContext;
import org.ogf.graap.wsag.api.security.KeystoreProperties;
import org.ogf.graap.wsag.api.types.AgreementOfferType;
import org.ogf.graap.wsag.client.api.AgreementClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryRegistryClient;
import org.ogf.graap.wsag.it.AbstractIntegrationTest;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.ServiceDescriptionTermType;
import org.ogf.schemas.graap.wsAgreement.TerminateInputType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * Code snippets used in documentation.
 * 
 * @author Karl Catewicz
 */
public class AbstractAgreementIntegrationSnippets
{

    /**
     * number of factories configured in the framework
     */
    public static final int EXPECTED_FACTORIES = 2;

    /**
     * number of templates at factory 1
     */
    public static final int EXPECTED_TEMPLATES_FACTORY_1 = 4;

    /**
     * Returns the login context used in the unit tests.
     * 
     * @return the client login context
     * 
     * @throws Exception
     *             indicates a failure during test case execution
     */
    public static LoginContext getLoginContext() throws Exception
    {
        /*
         * create a keystore login context
         */
        // START SNIPPET: CreateLoginContext
        KeystoreProperties properties = new KeystoreProperties();
        properties.setKeyStoreAlias( "wsag4j-user" );
        properties.setPrivateKeyPassword( "user@wsag4j" );

        properties.setKeyStoreType( "JKS" );
        properties.setKeystoreFilename( "/wsag4j-client-keystore.jks" );
        properties.setKeystorePassword( "user@wsag4j" );

        properties.setTruststoreType( "JKS" );
        properties.setTruststoreFilename( "/wsag4j-client-keystore.jks" );
        properties.setTruststorePassword( "user@wsag4j" );

        LoginContext loginContext = new KeystoreLoginContext( properties );
        loginContext.login();
        // END SNIPPET: CreateLoginContext

        return loginContext;
    }

    /**
     * Code snippet to illustrate how templates are retrieved from an agreement factory.
     * 
     * @throws Exception
     *             indicates a failure during test case execution
     */
    @SuppressWarnings( "unused" )
    public void testGetTemplate() throws Exception
    {

        AgreementFactoryClient[] factories = getAgreementFactoryClients();
        //
        // We have 2 factories installed. One returns 1 template, the other 2.
        //

        AgreementFactoryClient factory = null;
        for ( int i = 0; i < factories.length; i++ )
        {
            AgreementTemplateType[] templates = factories[i].getTemplates();
            if ( templates.length == EXPECTED_TEMPLATES_FACTORY_1 )
            {
                factory = factories[i];
            }
        }
        // START SNIPPET: GetTemplate
        AgreementTemplateType[] templates = factory.getTemplates();
        // END SNIPPET: GetTemplate
    }

    /**
     * Code snippet to illustrate how agreements are created with an agreement factory.
     * 
     * @throws Exception
     *             indicates a failure during test case execution
     */
    public void testCreateAgreement() throws Exception
    {

        AgreementFactoryClient[] factories = getAgreementFactoryClients();

        //
        // lookup factory with resource id "SAMPLE-INSTANCE-1"
        //
        // START SNIPPET: LookupFactoryWithID
        AgreementFactoryClient factory = null;
        for ( int i = 0; i < factories.length; i++ )
        {
            if ( "SAMPLE-INSTANCE-1".equals( factories[i].getResourceId() ) )
            {
                factory = factories[i];
                break;
            }
        }
        // END SNIPPET: LookupFactoryWithID
        //
        // retrieve template with name "SAMPLE1" and id "1" from the factory and print it on the console
        //
        // START SNIPPET: RetrieveTemplateWithID
        AgreementTemplateType template = factory.getTemplate( "SAMPLE1", "1" );
        // END SNIPPET: RetrieveTemplateWithID

        // START SNIPPET: CreateAgreement

        //
        // create the agreement offer based on the selected template
        //
        AgreementOffer offer = new AgreementOfferType( template );

        //
        // get the service description term from the offer
        //
        ServiceDescriptionTermType sdt = offer.getTerms().getAll().getServiceDescriptionTermArray( 0 );

        //
        // select the service description
        //
        // The JobDescriptionDocument is defined as a document element. We use the
        // JobDescriptionDocument QName to select the JobDescriptionDocument from the SDT.
        //
        JobDescriptionType job =
            (JobDescriptionType) sdt.selectChildren( JobDescriptionDocument.type.getDocumentElementName() )[0];

        //
        // changes to the job object are automatically reflected in the offer
        //
        job.getResources().addNewIndividualCPUCount().addNewExact().setDoubleValue( 4 );

        //
        // now create the agreement instance
        //
        AgreementClient agreement = factory.createAgreement( offer );

        // END SNIPPET: CreateAgreement

        agreement.destroy();
    }

    /**
     * Code snippet to illustrate exception handling during agreements creation.
     * 
     * @throws Exception
     *             indicates a failure during test case execution
     */
    @SuppressWarnings( "unused" )
    public void testCreateAgreementException() throws Exception
    {

        AgreementFactoryClient[] factories = getAgreementFactoryClients();

        AgreementFactoryClient factory = factories[0];
        AgreementTemplateType[] templates = factory.getTemplates();

        AgreementOffer offer = new AgreementOfferType( templates[0] );
        offer.getTerms().getAll().getServiceDescriptionTermArray( 0 ).setName( "TEST_EXCEPTION" );

        // START SNIPPET: CreateAgreementException
        AgreementClient agreement = null;
        AgreementFactoryException exception = null;
        try
        {
            agreement = factory.createAgreement( offer );
        }
        catch ( AgreementFactoryException e )
        {
            exception = e;
        }
        // END SNIPPET: CreateAgreementException
    }

    /**
     * Code snippet to illustrate agreements termination.
     * 
     * @throws Exception
     *             indicates a failure during test case execution
     */
    public void testCreateTerminateAgreement() throws Exception
    {

        AgreementFactoryClient[] factories = getAgreementFactoryClients();

        AgreementFactoryClient factory = factories[1];
        AgreementTemplateType[] templates = factory.getTemplates();

        AgreementOffer offer = new AgreementOfferType( templates[0] );
        AgreementClient agreement = factory.createAgreement( offer );

        // START SNIPPET: TerminateAgreement
        agreement.terminate( TerminateInputType.Factory.newInstance() );
        // END SNIPPET: TerminateAgreement

        agreement.destroy();
    }

    /**
     * Returns the factory clients for this tests.
     * 
     * @return the factory clients
     * 
     * @throws Exception
     *             indicates an error while generating the agreement factory client list
     */
    public static AgreementFactoryClient[] getAgreementFactoryClients() throws Exception
    {
        AgreementFactoryClient[] factories = null;

        LoginContext loginContext = AbstractIntegrationTest.getLoginContext();

        // START SNIPPET: ListAgreementFactories
        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
        epr.addNewAddress().setStringValue( "http://127.0.0.1:8080/wsag4j" );

        AgreementFactoryRegistryClient registry =
            AgreementFactoryRegistryClient.FACTORY.newInstance( epr, loginContext );

        factories = registry.listAgreementFactories();
        // END SNIPPET: ListAgreementFactories

        return factories;
    }

}
