/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.it;

import org.apache.log4j.Logger;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDescriptionDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDescriptionType;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.types.AgreementOfferType;
import org.ogf.graap.wsag.client.api.AgreementClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryRegistryClient;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermStateDefinition;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermStateType;
import org.ogf.schemas.graap.wsAgreement.ServiceDescriptionTermType;
import org.ogf.schemas.graap.wsAgreement.TerminateInputType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * AbstractAgreementFactoryTest
 * 
 * @author Oliver Waeldrich
 * 
 */
public abstract class AbstractAgreementFactoryTest extends AbstractIntegrationTest
{

    protected static final Logger LOG = Logger.getLogger( AbstractAgreementFactoryTest.class );

    /**
     * @param name
     *            the test name
     */
    public AbstractAgreementFactoryTest( String name )
    {
        super( name );
    }

    /**
     * Tests creation and destruction of agreement WSRF resources.
     */
    public void testCreateDestroyAgreement()
    {
        try
        {
            LOG.info( "Entering TestCase: testCreateDestroyAgreement" );

            AgreementFactoryClient[] factories = getAgreementFactoryClients();
            assertEquals( getDefaultFactoryCount(), factories.length );

            AgreementFactoryClient factory = factories[0];

            AgreementClient[] agreements;

            /*
             * log.info("Check the number of existing agreements. Excpected <0>"); AgreementClient[]
             * agreements = factory.listAgreements();
             * 
             * for (int i = 0; i < agreements.length; i++) { log.info(agreements[i].getEndpoint().xmlText(new
             * XmlOptions().setSavePrettyPrint())); }
             * 
             * if (agreements.length > 0) { System.exit(0); } assertEquals(0, agreements.length);
             */

            LOG.info( "Creating new Agreements" );
            AgreementTemplateType[] templates = factory.getTemplates();
            AgreementOffer offer = new AgreementOfferType( templates[0] );

            LOG.info( "Create agreement 1" );
            AgreementClient agreement1 = factory.createAgreement( offer );

            assertNotNull( "the created agreement must not be <null>", agreement1 );

            agreements = factory.listAgreements();
            assertEquals( 1, agreements.length );

            LOG.info( "Create agreement 2" );
            AgreementClient agreement2 = factory.createAgreement( offer );
            assertNotNull( "the created agreement must not be <null>", agreement2 );

            LOG.info( "List agreements" );
            agreements = factory.listAgreements();
            assertEquals( 2, agreements.length );

            LOG.info( "Destroy agreement 1" );
            agreement1.destroy();
            agreements = factory.listAgreements();
            assertEquals( 1, agreements.length );

            LOG.info( "Destroy agreement 2" );
            agreement2.destroy();
            agreements = factory.listAgreements();
            assertEquals( 0, agreements.length );
        }
        catch ( AgreementFactoryException e )
        {
            fail( "AgreementFactoryException: " + e.getMessage() );
        }
        catch ( ResourceUnavailableException e )
        {
            fail( "ResourceUnavailableException: " + e.getMessage() );
        }
        catch ( ResourceUnknownException e )
        {
            fail( "ResourceUnknownException: " + e.getMessage() );
        }
        catch ( Exception e )
        {
            fail( "Could not create agreement factory instance. Error: " + e.getMessage() );
        }
    }

    /**
     * Tests retrieving templates from an agreement factory.
     * 
     * @throws Exception
     *             indicates a test failure
     */
    public void testGetTemplate() throws Exception
    {
        LOG.info( "Entering TestCase: testGetTemplate" );

        // START SNIPPET: GetTemplate
        AgreementFactoryClient[] factories = getAgreementFactoryClients();
        assertEquals( getDefaultFactoryCount(), factories.length );

        //
        // We have 2 factories installed. One returns 1 template, the other 2.
        //
        AgreementFactoryClient factory = null;
        for ( int i = 0; i < factories.length; i++ )
        {
            AgreementTemplateType[] templates = factories[i].getTemplates();

            assertNotNull( templates );

            if ( templates.length == EXPECTED_TEMPLATES_FACTORY_1 )
            {
                factory = factories[i];
            }
        }

        assertNotNull( factory );

        // factory.setTrace(true);

        AgreementTemplateType[] templates = factory.getTemplates();
        // END SNIPPET: GetTemplate

        assertNotNull( "check that the call returned a template array", templates );

        assertEquals( "check that the call returned exactly " + EXPECTED_TEMPLATES_FACTORY_1 + " templates",
            EXPECTED_TEMPLATES_FACTORY_1, templates.length );

        //
        // Here, we assume that the templates are returned in the same order
        // as they were added to the server. WSAG4J supports this behavior.
        //
        assertEquals( "check that the name of the 1. template equals \"SAMPLE1\"", "SAMPLE1",
            templates[0].getName() );
        assertEquals( "check that the id of the 1. template equals \"1\"", "1", templates[0].getTemplateId() );
        assertEquals( "check that the name of the 2. template equals \"SAMPLE2\"", "SAMPLE2",
            templates[1].getName() );
        assertEquals( "check that the agreement id of the 2. template equals \"2\"", "2",
            templates[1].getTemplateId() );
        assertEquals( "check that the name of the 1. template equals \"sample_service_description_term\"",
            "SAMPLE1_SDT", templates[0].getTerms().getAll().getServiceDescriptionTermArray( 0 ).getName() );
        assertEquals( "check that the service name of the 1. template equals \"sample_service\"",
            "SAMPLE1_SERVICE", templates[0].getTerms().getAll().getServiceDescriptionTermArray( 0 )
                                           .getServiceName() );

    }

    /**
     * Tests agreement creation.
     * 
     * @throws Exception
     *             indicates a test failure
     */
    public void testCreateAgreement() throws Exception
    {
        LOG.info( "Entering TestCase: testCreateAgreement" );

        // START SNIPPET: CreateAgreement

        AgreementFactoryClient[] factories = getAgreementFactoryClients();
        assertEquals( getDefaultFactoryCount(), factories.length );

        //
        // lookup factory with resource id "SAMPLE-INSTANCE-1"
        //
        AgreementFactoryClient factory = null;
        for ( int i = 0; i < factories.length; i++ )
        {
            if ( "SAMPLE-INSTANCE-1".equals( factories[i].getResourceId() ) )
            {
                factory = factories[i];
                break;
            }
        }

        //
        // retrieve template with name "SAMPLE1" and id "1" from the factory and print it on the console
        //
        AgreementTemplateType template = factory.getTemplate( "SAMPLE1", "1" );
        // log.info("TEMPLATE:\n" + template.xmlText());

        //
        // create the agreement offer based on the selected template
        //
        AgreementOffer offer = new AgreementOfferType( template );

        //
        // get the service description term from the offer
        //
        ServiceDescriptionTermType sdt = offer.getTerms().getAll().getServiceDescriptionTermArray( 0 );

        //
        // select the service description
        //
        // The JobDescriptionDocument is defined as a document element. We use the
        // JobDescriptionDocument QName to select the JobDescriptionDocument from the SDT.
        //
        JobDescriptionType job =
            (JobDescriptionType) sdt.selectChildren( JobDescriptionDocument.type.getDocumentElementName() )[0];

        //
        // changes to the job object are automatically reflected in the offer
        //
        job.getResources().addNewIndividualCPUCount().addNewExact().setDoubleValue( 4 );

        // print the resulting offer to the console
        // log.info("AGREEMENT OFFER:\n" + offer.getXMLObject());

        //
        // now create the agreement instance
        //
        AgreementClient agreement = factory.createAgreement( offer );

        // END SNIPPET: CreateAgreement

        assertNotNull( "the created agreement must not be <null>", agreement );

        GuaranteeTermStateType[] guaranteeTermStates = agreement.getGuaranteeTermStates();
        assertEquals( "the number of guarantee term states must be <2>", guaranteeTermStates.length, 2 );

        assertEquals( "term_0", guaranteeTermStates[0].getTermName() );
        assertEquals( "term_1", guaranteeTermStates[1].getTermName() );

        assertEquals( GuaranteeTermStateDefinition.INT_NOT_DETERMINED, guaranteeTermStates[0].getState()
                                                                                             .intValue() );
        assertEquals( GuaranteeTermStateDefinition.INT_FULFILLED, guaranteeTermStates[1].getState()
                                                                                        .intValue() );

        agreement.destroy();
    }

    /**
     * Tests agreement creation and termination.
     * 
     * @throws Exception
     *             indicates a test failure
     */
    public void testCreateTerminateAgreement() throws Exception
    {
        LOG.info( "Entering TestCase: testCreateTerminateAgreement" );

        AgreementFactoryClient[] factories = getAgreementFactoryClients();
        assertEquals( getDefaultFactoryCount(), factories.length );

        for ( int i = 0; i < getDefaultFactoryCount(); i++ )
        {
            LOG.info( "+ " + factories[i].getResourceId() );
            LOG.info( "   + Number of templates: " + factories[i].getTemplates().length );
        }

        AgreementFactoryClient factory = factories[0];
        AgreementTemplateType[] templates = factory.getTemplates();

        AgreementOffer offer = new AgreementOfferType( templates[0] );
        AgreementClient agreement = factory.createAgreement( offer );
        assertNotNull( agreement );

        AgreementClient[] agreementClients = factory.listAgreements();
        assertNotNull( agreementClients.length );
        assertEquals( "agreement client not available in registry", 1, agreementClients.length );

        // START SNIPPET: TerminateAgreement
        agreement.terminate( TerminateInputType.Factory.newInstance() );
        // END SNIPPET: TerminateAgreement

        agreement.destroy();

        agreementClients = factory.listAgreements();
        assertNotNull( agreementClients.length );
        assertEquals( "agreement client not destroyed", 0, agreementClients.length );
    }

    /**
     * Tests exception handling in the agreement creation process. This includes proper de-/serialization of
     * faults.
     * 
     * @throws Exception
     *             indicates a test failure
     */
    public void testCreateAgreementException() throws Exception
    {
        LOG.info( "Entering TestCase: testCreateAgreementException" );

        AgreementFactoryClient[] factories = getAgreementFactoryClients();
        assertEquals( getDefaultFactoryCount(), factories.length );

        AgreementFactoryClient factory = factories[0];
        AgreementTemplateType template = factory.getTemplate( "SAMPLE3", "1" );

        AgreementOffer offer = new AgreementOfferType( template );
        offer.getTerms().getAll().getServiceDescriptionTermArray( 0 ).setName( "TEST_EXCEPTION" );

        // START SNIPPET: CreateAgreementException
        AgreementClient agreement = null;
        AgreementFactoryException exception = null;
        try
        {
            agreement = factory.createAgreement( offer );
        }
        catch ( AgreementFactoryException e )
        {
            exception = e;
        }
        // END SNIPPET: CreateAgreementException

        assertNull( agreement );
        assertNotNull( exception );
    }

    /**
     * Test for Bug 79.
     * 
     * Bug 79: There seems to be a problem in creating agreement factory if the URL contains "/" at the end of
     * the address. It works fine if one uses the following url string:
     * 
     * http://localhost:8080/wsag4j-agreement-factory-0.1.14
     * 
     */
    public void testGetFactoryClientURL()
    {
        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
        epr.addNewAddress().setStringValue( getUrl() + "/" );

        AgreementFactoryRegistryClient registry = null;
        AgreementFactoryClient[] factories = null;

        try
        {
            registry = AgreementFactoryRegistryClient.FACTORY.newInstance( epr, getLoginContext() );
            factories = registry.listAgreementFactories();
            assertEquals( getDefaultFactoryCount(), factories.length );
        }
        catch ( Exception ex )
        {
            fail( "Failed to initialize factory list (BUG 79). " + ex.getMessage() );
        }

        try
        {
            AgreementTemplateType[] templates = factories[0].getTemplates();
            assertTrue( templates.length > 0 );

        }
        catch ( Exception ex )
        {
            fail( "Failed to list agreement templates  (BUG 79). " + ex.getMessage() );
        }
    }

    /**
     * Test for accessing unknown agreement WSRF resources.
     * 
     * @throws Exception
     *             indicates a test failure
     */
    public abstract void testResourceUnknown() throws Exception;
}
