/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.it;

import java.util.Calendar;
import java.util.GregorianCalendar;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.NegotiationException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.types.AgreementOfferType;
import org.ogf.graap.wsag.client.api.AgreementClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryClient;
import org.ogf.graap.wsag.client.api.NegotiationClient;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationConstraintSectionType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferContextType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferStateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationRoleType;

/**
 * Default unit test implementation of the WS-Agreement Negotiation port type.
 * 
 * @author hrasheed
 * 
 */
public abstract class AbstractAgreementNegotiationTest extends AbstractIntegrationTest
{

    private static final Logger LOG = Logger.getLogger( AbstractAgreementNegotiationTest.class );

    private static final String FACTORY_RESOURCE_ID = "SAMPLE-INSTANCE-1";

    private static final String TEMPLATE_NAME = "SAMPLE1";

    /**
     * 
     * @param name
     *            the test name
     */
    public AbstractAgreementNegotiationTest( String name )
    {
        super( name );
    }

    /**
     * Tests the negotiation of offers and counter offers.
     * 
     * @throws Exception
     *             indicates a test failure
     */
    public void testAgreementNegotiation() throws Exception
    {

        NegotiationClient negotiation = initiateNegotiation();

        try
        {

            LOG.info( "getting NegotionContext" );

            NegotiationContextType context = negotiation.getNegotiationContext();
            assertNotNull( context );
            if ( LOG.isTraceEnabled() )
            {
                LOG.trace( "negotiationContext: " + context.xmlText() );
            }

            //
            // retrieve the agreement templates for which negotiation is supported
            //

            LOG.info( "getting negotiable templates" );

            AgreementTemplateType[] negotiableTemplates = negotiation.getNegotiableTemplates();
            assertNotNull( negotiableTemplates );
            if ( LOG.isTraceEnabled() )
            {
                LOG.trace( "negotiable templates:" );
                for ( int i = 0; i < negotiableTemplates.length; i++ )
                {
                    LOG.trace( negotiableTemplates[i].xmlText() );
                }
            }

            //
            // negotiate an acceptable agreement offer
            //

            LOG.info( "negotiating with negotiation instance" );

            AgreementTemplateType template = null;
            for ( int i = 0; i < negotiableTemplates.length; i++ )
            {
                AgreementTemplateType agreementTemplate = negotiableTemplates[i];
                if ( agreementTemplate.getName().equals( TEMPLATE_NAME ) )
                {
                    template = agreementTemplate;
                }
            }

            assertEquals( TEMPLATE_NAME, template.getName() );
            if ( LOG.isTraceEnabled() )
            {
                LOG.trace( "tempalte to negotiate: " + template.xmlText() );
            }

            NegotiationOfferType negotiationOfferType = NegotiationOfferType.Factory.newInstance();
            negotiationOfferType.setName( template.getContext().getTemplateName() );
            negotiationOfferType.setAgreementId( template.getContext().getTemplateId() );
            negotiationOfferType.addNewContext().set( template.getContext() );
            negotiationOfferType.addNewTerms().set( template.getTerms() );

            NegotiationOfferContextType negOfferContext = NegotiationOfferContextType.Factory.newInstance();
            negOfferContext.setExpirationTime( Calendar.getInstance() );
            negOfferContext.setCreator( NegotiationRoleType.NEGOTIATION_INITIATOR );
            NegotiationOfferStateType negOfferState = NegotiationOfferStateType.Factory.newInstance();
            negOfferState.addNewAdvisory();
            negOfferContext.setState( negOfferState );

            String templateId = template.getTemplateId() + "-" + template.getName();
            negOfferContext.setCounterOfferTo( templateId );

            NegotiationConstraintSectionType constraints =
                NegotiationConstraintSectionType.Factory.newInstance();
            constraints.addNewConstraint();

            negotiationOfferType.setNegotiationOfferContext( negOfferContext );
            negotiationOfferType.setNegotiationConstraints( constraints );
            negotiationOfferType.setOfferId( templateId );

            NegotiationOfferType[] negotiationOfferTypes = { negotiationOfferType };
            if ( LOG.isTraceEnabled() )
            {
                LOG.trace( "negotiation offer: " + negotiationOfferTypes[0].xmlText() );
            }

            NegotiationOfferType[] counterOffers = negotiation.negotiate( negotiationOfferTypes );
            assertNotNull( counterOffers );
            assertEquals( 1, counterOffers.length );

            NegotiationOfferType counterOffer = counterOffers[0];
            if ( LOG.isTraceEnabled() )
            {
                LOG.trace( "counter offer: " + counterOffer.xmlText() );
            }

            assertNotNull( counterOffer.getOfferId() );
            assertFalse( counterOffer.getOfferId().equals( "" ) );
            assertNotNull( counterOffer.getNegotiationOfferContext() );
            assertNotNull( counterOffer.getNegotiationConstraints() );
            assertNotNull( counterOffer.getContext() );
            assertNotNull( counterOffer.getTerms() );
            assertNotNull( negotiationOfferType.getOfferId(), counterOffer.getNegotiationOfferContext()
                                                                          .getCounterOfferTo() );
            assertNotNull( counterOffer.getAgreementId() );
            assertEquals( template.getName(), counterOffer.getName() );

            NegotiationOfferType[] offers = negotiation.getNegotiationOffers();
            assertNotNull( offers );
            if ( LOG.isTraceEnabled() )
            {
                LOG.trace( "valid offers[0]: " + offers[0].xmlText() );
            }

            //
            // create agreement offer and create agreement
            //

            AgreementOffer offer = new AgreementOfferType( offers[0] );
            AgreementClient agreement = getFactoryClient().createAgreement( offer );
            assertNotNull( agreement );

            //
            // finally terminate the negotiation process
            //
            LOG.info( "terminating agreement negotiation" );
            negotiation.terminate();

        }
        catch ( NegotiationException e )
        {
            fail( "NegotiationException: " + e.getMessage() );
        }
        catch ( ResourceUnavailableException e )
        {
            fail( "ResourceUnavailableException: " + e.getMessage() );
        }
        catch ( ResourceUnknownException e )
        {
            fail( "ResourceUnknownException: " + e.getMessage() );
        }
        catch ( Exception e )
        {
            fail( "negotiation test fails. Error: " + e.getMessage() );
        }
    }

    private NegotiationClient initiateNegotiation()
    {
        AgreementFactoryClient factory = null;
        NegotiationClient negotiation = null;

        try
        {

            factory = getFactoryClient();

            NegotiationContextDocument negContextDoc = NegotiationContextDocument.Factory.newInstance();
            NegotiationContextType negContext = negContextDoc.addNewNegotiationContext();
            negContext.setAgreementResponder( NegotiationRoleType.NEGOTIATION_RESPONDER );
            negContext.setExpirationTime( new GregorianCalendar() );

            negContext.addNewNegotiationType().addNewNegotiation();

            negotiation = factory.initiateNegotiation( negContext );
            assertNotNull( "the created negotiatin client is null", negotiation );

            LOG.info( "negotiation instance is created successfully" );

        }
        catch ( NegotiationFactoryException e )
        {
            fail( "NegotiationFactoryException: " + e.getMessage() );
        }
        catch ( ResourceUnavailableException e )
        {
            fail( "ResourceUnavailableException: " + e.getMessage() );
        }
        catch ( ResourceUnknownException e )
        {
            fail( "ResourceUnknownException: " + e.getMessage() );
        }
        catch ( Exception e )
        {
            fail( "Could not create negotiation client instance. Error: " + e.getMessage() );
        }

        return negotiation;
    }

    /**
     * @return
     * @throws ResourceUnknownException
     * @throws ResourceUnavailableException
     */
    private AgreementFactoryClient getFactoryClient()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementFactoryClient factory;
        //
        // First create a negotiation instance. The instance is created
        // based on a negotiation context that specifies the type of the
        // negotiation process (NEGOTIATION) and the negotiation participators.
        //
        AgreementFactoryClient[] factories = getAgreementFactoryClients();
        assertEquals( getDefaultFactoryCount(), factories.length );

        if ( factories[0].getResourceId().equals( FACTORY_RESOURCE_ID ) )
        {
            factory = getAgreementFactoryClients()[0];
        }
        else
        {
            factory = getAgreementFactoryClients()[1];
        }
        return factory;
    }
}
