/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.it;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.types.AgreementOfferType;
import org.ogf.graap.wsag.client.api.AgreementClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryClient;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;

/**
 * AbstractAgreementFactoryTest
 * 
 * @author Oliver Waeldrich
 * 
 */
public abstract class AbstractAgreementRegistryTest extends AbstractIntegrationTest
{

    @SuppressWarnings( "unused" )
    private static final Logger LOG = Logger.getLogger( AbstractAgreementRegistryTest.class );

    /**
     * 
     * @param name
     *            the test name
     */
    public AbstractAgreementRegistryTest( String name )
    {
        super( name );
    }

    /**
     * Tests adding and removing agreements from/to the agreement service group.
     */
    public void testFirstRegistryAddRemove()
    {
        try
        {
            AgreementFactoryClient[] factories = getAgreementFactoryClients();
            assertTrue( "expected at least one registry to test", 1 <= getDefaultFactoryCount() );
            assertEquals( "expected number of factory mismatch", getDefaultFactoryCount(), factories.length );

            AgreementFactoryClient factory1 = factories[0];
            assertEquals( 0, factory1.listAgreements().length );

            AgreementTemplateType[] templates1 = factory1.getTemplates();
            AgreementOffer offer1 = new AgreementOfferType( templates1[0] );

            factory1.createAgreement( offer1 );
            assertEquals( 1, factory1.listAgreements().length );

            factory1.createAgreement( offer1 );
            assertEquals( 2, factory1.listAgreements().length );

            factory1.listAgreements()[0].destroy();
            assertEquals( 1, factory1.listAgreements().length );

            factory1.listAgreements()[0].destroy();
            assertEquals( 0, factory1.listAgreements().length );
        }
        catch ( AgreementFactoryException e )
        {
            fail( "test failed due to a AgreementFactoryException\n" + e.getMessage() );
        }
        catch ( ResourceUnknownException e )
        {
            fail( "test failed due to a ResourceUnknownException\n" + e.getMessage() );
        }
        catch ( ResourceUnavailableException e )
        {
            fail( "test failed due to a ResourceUnavailableException\n" + e.getMessage() );
        }
        catch ( Exception e )
        {
            fail( "AgreementFactoryLocator.getFactory() failed due to a Exception\n" + e.getMessage() );
        }
    }

    /**
     * Tests listing agreement instances registered in an agreement service group.
     */
    public void testListAgreements()
    {
        try
        {
            AgreementFactoryClient[] factories = getAgreementFactoryClients();
            assertEquals( getDefaultFactoryCount(), factories.length );

            AgreementFactoryClient factory = factories[0];
            AgreementClient[] agreements = factory.listAgreements();
            // END SNIPPET: ListAgreements

            AgreementTemplateType[] templates = factory.getTemplates();

            assertNotNull( "the registered agreements must not be <null>", agreements );
            assertEquals( "the number of registered agreements <0>", 0, agreements.length );

            AgreementOffer offer = new AgreementOfferType( templates[0] );
            AgreementClient agreement = factory.createAgreement( offer );

            assertNotNull( "the created agreement must not be <null>", agreement );

            agreements = factory.listAgreements();
            assertNotNull( "the registered agreements must not be <null>", agreements );
            assertEquals( "the number of registered agreements must be <1>", 1, agreements.length );

            assertEquals( "The name of the agreement must match <SampleAgreement>", "SampleAgreement",
                agreements[0].getName() );

            offer = new AgreementOfferType( templates[0] );
            agreement = factory.createAgreement( offer );

            assertNotNull( "the created agreement must not be <null>", agreement );

            agreements = factory.listAgreements();
            assertNotNull( "the registered agreements must not be <null>", agreements );
            assertEquals( "the number of registered agreements must be <2>", 2, agreements.length );

            assertEquals( "The name of the agreement must match <SampleAgreement>", "SampleAgreement",
                agreements[0].getName() );
            assertEquals( "The name of the agreement must match <SampleAgreement>", "SampleAgreement",
                agreements[1].getName() );

        }
        catch ( AgreementFactoryException e )
        {
            fail( "test failed due to a AgreementFactoryException\n" + e.getMessage() );
        }
        catch ( ResourceUnknownException e )
        {
            fail( "test failed due to a ResourceUnknownException\n" + e.getMessage() );
        }
        catch ( ResourceUnavailableException e )
        {
            fail( "test failed due to a ResourceUnavailableException\n" + e.getMessage() );
        }
        catch ( Exception e )
        {
            fail( "AgreementFactoryLocator.getFactory() failed due to a Exception\n" + e.getMessage() );
        }
    }
}
